#!/usr/bin/env ruby -w
# ex:ts=10000
# vim:sts=4:sw=4:tw=0

require 'test/unit'
require 'eekmock'

class EekMock__test < Test::Unit::TestCase
    class Mocked
        attr_reader :method_reaches
        attr_reader :caught_method_missing_count
        def initialize
            @method_reaches = []
            @caught_missing_method_count = 0
        end
        def reached_haha() end
        def haha(*args)
            @method_reaches.push(args)
            reached_haha
        end
        def method_missing(symbol, *args, &block)
            symbol = symbol.to_s
            if symbol == "caught_missing_method"
                @caught_missing_method_count ++ 1
            else
                super
            end
        end
    end
    def test_proxy
        mocked = Mocked.new
        controller = EekMock.new(mocked)
        mock = controller.mock
        mock.haha(10000)
        mock.haha( 2000)
        assert_equal([[10000], [2000]], mocked.method_reaches)
        assert_raises(NoMethodError) {
            mock.no_such_method
        }
        assert_equal(0, controller.mock_count(:no_such_method), "Bad no_such_method count")
        assert_equal(2, controller.mock_count(:haha))
        # Test that we count the methods INSIDE the mocked object.
        assert_equal(2, controller.mock_count(:reached_haha))
        # FIXME Should test for mock_ignore_missing, and still counting
        # (If we really WANT mock_ignore_missing - I'm not sure that it is an
        # advantage if we already have the ability to mock a class)
        # FIXME Should test that method_missing work correctly for the mock
        assert_nothing_raised {
            mock.caught_missing_method
        }
        assert_equal(1, controller.mock_count(:method_missing))
        assert_equal(0, controller.mock_count(:caught_missing_method))
        assert_equal(1, mock.caught_method_missing_count)
    end
end
